import 'package:flutter/widgets.dart';
import 'package:flutter_custom_form_component/widget/field/date/224/mixin.dart';

class DdMmYyyyFieldWidget extends StatefulWidget {
  late final String _hint;
  late final bool _isValid;
  late final Function(int? year, int? month, int? day)? _onChanged;
  late final String _separator;

  DdMmYyyyFieldWidget(
      {String hint = 'dd - mm - yyyy',
      bool isValid = true,
      Function(int? year, int? month, int? day)? onChanged,
      String separator = ' - ',
      Key? key})
      : super(key: key) {
    _hint = hint;
    _isValid = isValid;
    _onChanged = onChanged;
    _separator = separator;
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<DdMmYyyyFieldWidget> with Date224StateMixin {
  @override
  Widget build(final BuildContext context) =>
      buildMixin(context, hint: widget._hint);

  @override
  void dispose() {
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    initStateMixin(initialValid: widget._isValid, separator: widget._separator);
  }

  @override
  void onChanged() {
    final List<String> strings = split(separator: widget._separator);
    year = int.tryParse(strings.length > 2 ? strings[2] : '');
    month = int.tryParse(strings.length > 1 ? strings[1] : '');
    day = int.tryParse(strings.isNotEmpty ? strings[0] : '');
    widget._onChanged?.call(year, month, day);
  }

  @override
  void refresh() => setState(() {});
}
