import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/app/widget.dart';
import 'package:flutter_custom_form_component/widget/field/date/224/formatter.dart';

mixin Date224StateMixin {
  final TextEditingController _controller = TextEditingController();
  int? _day, _month, _year;
  final FocusNode _focusNode = FocusNode();
  late final Date224Formatter _formatter;
  late final bool _initialValid;
  late bool _isValid;
  late final int _maxLength;

  Widget buildMixin(final BuildContext context, {required final String hint}) =>
      Container(
          child: Padding(
              child: TextField(
                  controller: _controller,
                  decoration: InputDecoration(
                      border: InputBorder.none,
                      counterText: '',
                      hintText: hint),
                  focusNode: _focusNode,
                  inputFormatters: [
                    FilteringTextInputFormatter.digitsOnly,
                    _formatter
                  ],
                  keyboardType: Platform.isAndroid
                      ? TextInputType.datetime
                      : TextInputType.number,
                  maxLength: _maxLength,
                  style: _initialValid && _isValid
                      ? null
                      : const TextStyle(color: colorRed)),
              padding: const EdgeInsets.symmetric(horizontal: 20.0)),
          decoration: BoxDecoration(
              border: Border.all(color: colorBlue, width: 0.5),
              borderRadius: BorderRadius.circular(8.0),
              color: colorWhite),
          height: double.infinity,
          width: double.infinity);

  void _controllerListener() {
    onChanged();
    if (_controller.text.length == _maxLength) {
      _focusNode.unfocus();
    }
  }

  void disposeMixin() {
    _controller.removeListener(_controllerListener);
    _controller.dispose();
    _focusNode.removeListener(_focusNodeListener);
    _focusNode.dispose();
  }

  void _focusNodeListener() {
    if (!_focusNode.hasFocus) {
      if (_controller.text.length < _maxLength) {
        _isValid = false;
        refresh();
      } else {
        validate();
      }
    }
  }

  void initStateMixin(
      {required final bool initialValid, required final String separator}) {
    _formatter = Date224Formatter(separator);
    _initialValid = initialValid;
    _isValid = initialValid;
    _maxLength = 8 + (separator.length * 2);
    _controller.addListener(_controllerListener);
    _focusNode.addListener(_focusNodeListener);
  }

  void onChanged();

  void refresh();

  List<String> split({required final String separator}) =>
      _controller.text.split(separator);

  void validate() {
    if (_year == null || _month == null || _day == null) {
      _isValid = false;
    } else {
      final DateTime date = DateTime(_year!, _month!, _day!);
      _isValid = date.year == _year && date.month == _month && date.day == _day;
    }
    refresh();
  }

  set day(final int? day) => _day = day;

  int? get day => _day;

  set month(final int? month) => _month = month;

  int? get month => _month;

  set year(final int? year) => _year = year;

  int? get year => _year;
}
